// clone of cable management system by Ivan Miranda
// for original design, see https://www.youtube.com/watch?v=Wn9LavVHeq8

// all sizes in mm, this is metric country

// Orientation: width = Y, height = X, length = Z

// === CONSTANTS ===

// what to render in the output - makes it easier to separate the parts for later export
RENDER_COVER = true;
RENDER_CLIP  = true;

// the outer dimensions of the cover
COVER_WIDTH  =  20;
COVER_HEIGHT =  20;
COVER_LENGTH = 100;

// the size and position of the "dimples" to hold the cover on the clips
DIMPLE_DIAMETER    = 2.0;
DIMPLE_EDGE_OFFSET = 1.5;

// the clearance to leave between clip and cover
CLEARANCE = 0.1;

// the radius of the rounded corners of the screw tab of the clip
CLIP_TAB_CORNER_RADIUS = 0.5;

// the size of the hole in the clip screw tab
CLIP_SCREW_HOLE_WIDTH  = 6.0;
CLIP_SCREW_HOLE_LENGTH = 4.1;

// the other dimensions of the clip
CLIP_TAB_LENGTH = 10;
CLIP_CABLE_HOLDER_LENGTH = 10;

// the offset and size of the cable way in the clip
CLIP_CABLE_WAY_HEIGHT = 6.0;
CLIP_CABLE_WAY_OFFSET = 1.0;

// the overall material thickness
WALL_SIZE = 2.0;

// a very tiny amount to correct for some very annoying rendering issue
EPSILON = 0.001;

// === DERIVED SIZES ===

function cover_center_cutout_width() = COVER_WIDTH - 2 * WALL_SIZE;

function clip_width() = COVER_WIDTH - 2 * WALL_SIZE - 2 * CLEARANCE;
function clip_height() = COVER_HEIGHT - WALL_SIZE - CLEARANCE;
function clip_length() = CLIP_TAB_LENGTH + CLIP_CABLE_HOLDER_LENGTH;

function clip_center_cutout_width() = clip_width() - 2 * WALL_SIZE;

// === MODULES ===

module cover() {
	linear_extrude(height = COVER_LENGTH) {
		difference() {
			// create the basic outer shape
			union() {
				translate([0, -COVER_WIDTH/2])
					square([COVER_HEIGHT/2, COVER_WIDTH]);
				translate([COVER_HEIGHT/2, 0])
					circle(d = COVER_WIDTH);
			}
			// minus the center cutout
			union() {
				difference() {
					// the overall cutout profile
					union() {
						translate([-EPSILON, -cover_center_cutout_width()/2])
							square([COVER_HEIGHT/2 + EPSILON, cover_center_cutout_width()]);
						translate([COVER_HEIGHT/2, 0])
							circle(d = cover_center_cutout_width());
					}
					// minus the "dimples" to hold the cover on the clips
					translate([DIMPLE_EDGE_OFFSET, -cover_center_cutout_width()/2])
						circle(d = DIMPLE_DIAMETER);
					translate([DIMPLE_EDGE_OFFSET, cover_center_cutout_width()/2])
						circle(d = DIMPLE_DIAMETER);
				}
			}
		}
	}
}

module clip() {
	difference() {
		// the clip consists of two parts - the cable holder and the tab that holds the screw
		union() {
			_clip_screw_tab();
			_clip_cable_holder();
		}
		// minus the grooves that the "dimples" will sit in
		translate([DIMPLE_EDGE_OFFSET, -cover_center_cutout_width()/2, -EPSILON])
			cylinder(d = DIMPLE_DIAMETER + CLEARANCE, h = clip_length() + 2 * EPSILON);
		translate([DIMPLE_EDGE_OFFSET, cover_center_cutout_width()/2, -EPSILON])
			cylinder(d = DIMPLE_DIAMETER + CLEARANCE, h = clip_length() + 2 * EPSILON);
	}
}

module _clip_screw_tab() {
	difference() {
		// the "body" of the clip
		hull() {
			// the rounded corners at the bottom
			translate([0, -(clip_width()/2 - CLIP_TAB_CORNER_RADIUS), CLIP_TAB_CORNER_RADIUS])
				rotate([0, 90, 0])
					cylinder(h=WALL_SIZE, r=CLIP_TAB_CORNER_RADIUS);
			translate([0, clip_width()/2 - CLIP_TAB_CORNER_RADIUS, CLIP_TAB_CORNER_RADIUS])
				rotate([0, 90, 0])
					cylinder(h=WALL_SIZE, r=CLIP_TAB_CORNER_RADIUS);
			// the straight edge at the top
			translate([0, -clip_width()/2, CLIP_TAB_LENGTH - 1])
				cube([WALL_SIZE, clip_width(), 1]);
		}
		// minus the screw hole
		hull() {
			translate([-EPSILON, -CLIP_SCREW_HOLE_WIDTH/2, CLIP_TAB_LENGTH/2])
				rotate([0, 90, 0])
					cylinder(d = CLIP_SCREW_HOLE_LENGTH, h = WALL_SIZE + 2 * EPSILON);
			translate([-EPSILON, CLIP_SCREW_HOLE_WIDTH/2, CLIP_TAB_LENGTH/2])
				rotate([0, 90, 0])
					cylinder(d = CLIP_SCREW_HOLE_LENGTH, h = WALL_SIZE + 2 * EPSILON);
		}
	}
}

module _clip_cable_holder() {
	// place the cable holder upwards of the screw tab
	translate([0, 0, CLIP_TAB_LENGTH]) {
		linear_extrude(height = CLIP_CABLE_HOLDER_LENGTH) {
			difference() {
				// create the basic outer shape
				union() {
					translate([0, -clip_width()/2])
						square([clip_height()/2, clip_width()]);
					translate([clip_height()/2, 0])
						circle(d = clip_width());
				}
				// minus the center cutout
				union() {
					translate([WALL_SIZE, -clip_center_cutout_width()/2])
						square([clip_height()/2 - WALL_SIZE, clip_center_cutout_width()]);
					translate([clip_height()/2, 0])
						circle(d = clip_center_cutout_width());
				}
				// minus the cable way
				translate([WALL_SIZE + CLIP_CABLE_WAY_OFFSET, clip_width()/2 - WALL_SIZE - EPSILON]) 
					square([CLIP_CABLE_WAY_HEIGHT, WALL_SIZE + 2 * EPSILON]);
			}
		}
	}
}

// === MAIN "PROGRAM" ===

// increase the rendering resolution globally
$fn = 96;

if (RENDER_COVER)
	cover();
if (RENDER_CLIP)
	clip();